/****************************************************/
/* File: cm.y                                       */
/* A C-Minus Yacc/Bison specification file          */
/* for a recognizer                                 */
/* K. Louden 03/22/05                               */
/* CS 153, Spring, 2006                             */
/****************************************************/
%{
#include <stdio.h>
extern FILE* yyin;
%}

/* C-Minus multicharacter tokens */

%token ELSE IF INT RETURN VOID WHILE
%token ID NUM LE GE EQ NE

%% /* Grammar for C-Minus */

program     : decl_list
            ;

decl_list   : decl_list decl
            | decl
            ;

decl        : var_decl
            | fun_decl
            ;

var_decl    : type_spec ID ';'
            | type_spec ID '[' NUM ']' ';'
            ;
            
type_spec   : INT
            | VOID
            ;

fun_decl    : type_spec ID '(' params ')' cmpd_stmt
            ;

params      : param_list
            | VOID
            ;

param_list  : param_list ',' param
            | param
            ;

param       : type_spec ID
            | type_spec ID '[' ']'
            ;

cmpd_stmt   : '{' local_decls stmt_list '}'
            ;

local_decls : local_decls var_decl
            | /* empty */
            ;

stmt_list   : stmt_list stmt
            | /* empty */
            ;

stmt        : expr_stmt
            | cmpd_stmt
            | sel_stmt
            | iter_stmt
            | return_stmt
            ;

expr_stmt   : expr ';'
            | ';'
            ;

sel_stmt    : IF '(' expr ')' stmt
            | IF '(' expr ')' stmt ELSE stmt
            ;

iter_stmt   : WHILE '(' expr ')' stmt
            ;

return_stmt : RETURN ';'
            | RETURN expr ';'
            ;

expr        : var '=' expr
            | simple_expr
            ;

var         : ID
            | ID '[' expr ']'
            ;

simple_expr : add_expr relop add_expr
            | add_expr
            ;

relop       : LE 
            | '<'
            | '>'
            | GE 
            | EQ 
            | NE 
            ;

add_expr    : add_expr addop term
            | term
            ;

addop       : '+'
            | '-'
            ;

term        : term mulop factor 
            | factor { $$ = $1; }
            ;

mulop       : '*'
            | '/'
            ;

factor      : '(' expr ')'
            | var
            | call
            | NUM
            ;

call        : ID '(' args ')'
            ;

args        : arg_list
            | /* empty */
            ;

arg_list    : arg_list ','  expr
            | expr
            ;

%%

int yyerror(char * message)
{ printf(message);
  printf("\n");
  return 0;
}

int main(int argc, char* argv[])
{ if (argc != 2) 
  { fprintf(stderr,"usage: %s <filename>\n",argv[0]);
    exit(1);
  }
  yyin = fopen(argv[1],"r");
  if (yyin==NULL)
  { fprintf(stderr,"File %s not found\n",argv[1]);
    exit(1);
  }
  yyparse();
  fclose(yyin);
  return 0;
}

